@@ -18,12 +18,13 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  * $LastChangedBy: bastian $
- * $LastChangedDate: 2005-07-01 20:26:23 +0200 (Fr, 01 Jul 2005) $
- * $LastChangedRevision: 1009 $
+ * $LastChangedDate: 2008-03-23 21:55:27 +0100 (So, 23 Mär 2008) $
+ * $LastChangedRevision: 1581 $
  */
 
 #include <config.h>
 
+#include <errno.h>
 #include <limits.h>
 #include <stdlib.h>
 #include <string.h>
@@ -38,61 +39,38 @@
 #include "install.h"
 #include "log.h"
 #include "packages.h"
-#include "prepare.h"
 #include "suite.h"
+#include "target.h"
 
-char install_root[PATH_MAX];
-const char *helperdir;
+static const char *helperdir;
+static install_simulate_handler *simulate_handler;
 
-static int count, count_done;
-
-#define BUFSIZE_command 4096
-
-static void install_progress_set (int n)
-{
-  frontend_progress_set (((float) n/count) * 500 + 500);
-}
-
-static int install_execute_io_handler (const char *buf, size_t n __attribute__ ((unused)), void *user_data __attribute__ ((unused)))
-{
-  log_text (DI_LOG_LEVEL_OUTPUT, "%s", buf);
-  return 0;
-}
-
-static int install_execute_progress_update (di_packages *packages, char *package, di_package_status status)
+static void install_execute_progress_update (di_packages *packages, char *package, di_package_status status)
 {
   di_package *p = di_packages_get_package (packages, package, 0);
-  if (p)
+  if (p && p->status < status)
   {
-    if (p->status < status)
-    {
-      log_text (DI_LOG_LEVEL_DEBUG, "Updating %s to status %d", p->key.string, status);
-      p->status = status;
-    }
+    log_text (DI_LOG_LEVEL_DEBUG, "Updating %s to status %d", p->key.string, status);
+    p->status = status;
   }
-  else
-    return -1;
-  return 0;
 }
 
-static int install_execute_progress_io_handler (const char *buf, size_t n __attribute__ ((unused)), void *user_data)
+static int install_execute_progress_io_handler (char *buf, size_t n __attribute__ ((unused)), void *user_data)
 {
   di_packages *packages = user_data;
   char buf_package[128];
 
-  install_execute_io_handler (buf, n, NULL);
+  execute_io_log_handler (buf, n, NULL);
 
   if (sscanf (buf, "Unpacking replacement %128s", buf_package) == 1 ||
       sscanf (buf, "Unpacking %128s", buf_package) == 1)
   {
     log_message (LOG_MESSAGE_INFO_INSTALL_PACKAGE_UNPACK, buf_package);
-    install_progress_set (++count_done);
     install_execute_progress_update (packages, buf_package, di_package_status_unpacked);
   }
   else if (sscanf (buf, "Setting up %128s", buf_package) == 1)
   {
     log_message (LOG_MESSAGE_INFO_INSTALL_PACKAGE_CONFIGURE, buf_package);
-    install_progress_set (++count_done);
     install_execute_progress_update (packages, buf_package, di_package_status_installed);
   }
 
@@ -101,7 +79,9 @@ static int install_execute_progress_io_handler (const char *buf, size_t n __attr
 
 static int install_execute_target_progress (const char *command, di_packages *packages)
 {
-  return execute_target_full (command, install_execute_progress_io_handler, install_execute_io_handler, packages);
+  if (simulate_handler)
+    return simulate_handler (command);
+  return execute_target_full (command, install_execute_progress_io_handler, execute_io_log_handler, packages);
 }
 
 di_slist *install_list (di_packages *packages, di_packages_allocator *allocator, di_slist *install, di_package_priority priority, di_package_status status)
@@ -183,32 +163,14 @@ di_slist *install_list_package_only (di_packages *packages, char *package, di_pa
   return list;
 }
 
-int install_apt_cleanup (void)
-{
-  char buf[PATH_MAX];
-
-  strcpy (buf, install_root);
-  strcat (buf, "/etc/apt/sources.list");
-
-  if (truncate (buf, 0))
-    return 1;
-
-  strcpy (buf, "apt-get update");
-
-  if (execute_target (buf))
-    return 1;
-
-  return 0;
-}
-
 int install_apt_install (di_packages *packages, di_slist *install)
 {
-  char buf[BUFSIZE_command];
-  size_t len;
+  size_t size = 1024, len;
+  char *buf = malloc (size);
+  int count = 0, ret = 0;
   di_slist_node *node;
-  int count = 0;
 
-  strcpy (buf, "apt-get install --yes -o APT::Get::AllowUnauthenticated=true");
+  strcpy (buf, "apt-get install --yes -o APT::Get::AllowUnauthenticated=true -o APT::Install-Recommends=false");
   len = strlen (buf);
 
   for (node = install->head; node; node = node->next)
@@ -216,309 +178,212 @@ int install_apt_install (di_packages *packages, di_slist *install)
     count++;
     di_package *p = node->data;
     len += 1 + p->key.size;
-    if (len >= sizeof (buf))
-      log_text (DI_LOG_LEVEL_ERROR, "buffer overflow");
+    if (len >= size)
+    {
+      size *= 2;
+      buf = realloc (buf, size);
+    }
     strcat (buf, " ");
     strcat (buf, p->key.string);
   }
 
-  if (count && install_execute_target_progress (buf, packages))
-    return 1;
+  if (count)
+    ret = install_execute_target_progress (buf, packages);
 
-  return 0;
-}
+  free (buf);
 
-int install_apt_prepare (void)
-{
-  char buf[PATH_MAX];
-  FILE *out;
-
-  strcpy (buf, install_root);
-  strcat (buf, "/etc/apt/sources.list");
-
-  out = fopen (buf, "w");
-  if (!out)
-    return 1;
-
-  if (!fputs ("deb bootstrap: . main\n", out))
-    return 1;
-
-  if (fclose (out))
-    return 1;
-
-  if (execute_target ("apt-get update"))
-    return 1;
-
-  return 0;
-}
-
-int install_count_install (di_slist *install)
-{
-  int count = 0;
-  di_slist_node *node;
-
-  for (node = install->head; node; node = node->next)
-    count++;
-
-  return count;
-}
-
-static int install_dev_opt (char *buf, const char *name, mode_t mode, dev_t dev, int build)
-{
-  strcpy (buf, install_root);
-  strcat (buf, name);
-
-  unlink (buf);
-  if (build)
-    return mknod (buf, mode, dev);
-  return 0;
-}
-
-int install_dev_cleanup (void)
-{
-  char buf[PATH_MAX];
-
-  if (0)
-  {
-    install_dev_opt (buf, "/dev/null", 0, 0, 0);
-  }
-  return 0;
-}
-
-int install_dev_prepare (void)
-{
-  int ret = 0;
-  char buf[PATH_MAX];
-  mode_t mask;
-
-  mask = umask (0);
-  ret |= install_dev_opt (buf, "/dev/null", S_IFCHR | 0666, makedev (1, 3), 1);
-  umask (mask);
   return ret;
 }
 
 int install_dpkg_configure (di_packages *packages, int force)
 {
-  char buf[BUFSIZE_command];
+  char command[128] = "dpkg --configure -a";
 
-  strcpy (buf, "dpkg --configure -a");
   if (force)
-    strcat (buf, " --force-all");
+    strcat (command, " --force-all");
 
-  if (install_execute_target_progress (buf, packages))
-    return 1;
-
-  return 0;
+  return install_execute_target_progress (command, packages);
 }
 
-static int install_dpkg_all (char *buf, size_t bufsize, di_packages *packages, di_slist *install)
+static int install_dpkg_all (const char *command, di_packages *packages, di_slist *install)
 {
-  char buf1[256];
-  size_t len;
+  size_t size = 1024, len;
+  char *buf = malloc (size);
+  int count = 0, ret = 0;
   di_slist_node *node;
 
+  strcpy (buf, command);
   len = strlen (buf);
 
   for (node = install->head; node; node = node->next)
   {
+    count++;
     di_package *p = node->data;
+    char buf1[256];
     build_target_deb_root (buf1, sizeof (buf1), package_get_local_filename (p));
     len += 1 + strlen (buf1);
-    if (len >= bufsize)
-      log_text (DI_LOG_LEVEL_ERROR, "buffer overflow");
+    if (len >= size)
+    {
+      size *= 2;
+      buf = realloc (buf, size);
+    }
     strcat (buf, " ");
     strcat (buf, buf1);
   }
 
-  if (install_execute_target_progress (buf, packages))
-    return 1;
+  if (count)
+    ret = install_execute_target_progress (buf, packages);
 
-  return 0;
+  free (buf);
+
+  return ret;
 }
 
 int install_dpkg_install (di_packages *packages, di_slist *install, int force)
 {
-  char buf[BUFSIZE_command];
+  char command[128] = "dpkg -i";
 
-  strcpy (buf, "dpkg -i");
   if (force)
-    strcat (buf, " --force-all");
+    strcat (command, " --force-all");
 
-  return install_dpkg_all (buf, sizeof (buf), packages, install);
+  return install_dpkg_all (command, packages, install);
 }
 
 int install_dpkg_unpack (di_packages *packages, di_slist *install)
 {
-  char buf[BUFSIZE_command];
-
-  strcpy (buf, "dpkg --unpack --force-all");
-
-  return install_dpkg_all (buf, sizeof (buf), packages, install);
+  return install_dpkg_all ("dpkg --unpack --force-all", packages, install);
 }
 
 int install_extract (di_slist *install)
 {
-  int count = 0, count_done = 0;
   struct di_slist_node *node;
 
-  for (node = install->head; node; node = node->next)
-    count++;
-
   for (node = install->head; node; node = node->next)
   {
     di_package *p = node->data;
-    log_text (DI_LOG_LEVEL_MESSAGE, "Extracting %s", p->package);
+    log_message (LOG_MESSAGE_INFO_INSTALL_PACKAGE_EXTRACT, p->package);
     package_extract (p);
-    count_done++;
-    frontend_progress_set (((float) count_done/count) * 50 + 450);
   }
 
   return 0;
 }
 
-int install (di_packages *packages, di_packages_allocator *allocator, di_slist *install)
-{
-  di_slist_node *node;
-
-  if (chdir (install_root))
-    return 1;
-
-  for (node = install->head; node; node = node->next)
-    count += 2;
-
-  if (suite_install (packages, allocator, install))
-    log_text (DI_LOG_LEVEL_ERROR, "Couldn't install root!");
-
-  return 0;
-}
-
-int install_target_check (const char *target)
-{
-  struct stat statbuf;
-
-  umask (022);
-
-  if (!stat (target, &statbuf))
-  {
-    if (!(S_ISDIR (statbuf.st_mode)))
-      return 1;
-  }
-  else
-  {
-    if (mkdir (target, 0755))
-      return 1;
-  }
-
-  if (!realpath (target, install_root))
-    return 1;
-
-  if (install_root[0] != '/')
-  {
-    char tmp[PATH_MAX];
-    if (getcwd (tmp, sizeof (tmp) - strlen (install_root) - 1) == NULL)
-      return 1;
-    strcat (tmp, "/");
-    strcat (tmp, install_root);
-    if (!realpath (tmp, install_root))
-      return 1;
-  }
-  return 0;
-}
-
-int install_init (const char *_helperdir)
+int install_init (const char *_helperdir, install_simulate_handler _simulate_handler)
 {
   helperdir = _helperdir;
+  simulate_handler = _simulate_handler;
+
+  target_create_dir ("/var");
+  target_create_dir ("/var/lib");
+  target_create_dir ("/var/lib/dpkg");
+  target_create_file ("/var/lib/dpkg/available");
+  target_create_file ("/var/lib/dpkg/diversions");
+  target_create_file ("/var/lib/dpkg/status");
 
-  if (prepare_install ())
-    return 1;
   log_open ();
   return 0;
 }
 
-enum mount_status
+int install_mount (const char *what)
 {
-  umounted = 1,
-  mounted,
-};
+  char buf[PATH_MAX];
+  int ret = 0;
+  enum { TARGET_DEV, TARGET_PROC } target;
+
+  if (!strcmp (what, "dev"))
+    target = TARGET_DEV;
+  else if (!strcmp (what, "proc"))
+    target = TARGET_PROC;
+  else
+  {
+    log_text (DI_LOG_LEVEL_WARNING, "Unknown target for mount action: %s", what);
+    return 0;
+  }
+
+  if (simulate_handler)
+    return 0;
+
+  snprintf (buf, sizeof buf, "%s/%s", target_root, what);
+  switch (target)
+  {
+    case TARGET_DEV:
+      ret = mount ("/dev", buf, NULL, MS_BIND | MS_RDONLY, 0);
+      break;
+    case TARGET_PROC:
+      ret = mount ("proc", buf, "proc", 0, 0);
+      break;
+  }
+
+  if (ret)
+    log_text (DI_LOG_LEVEL_ERROR, "Failed to mount /%s: %s", what, strerror (errno));
 
-static enum mount_status mount_status_proc (enum mount_status status)
-{
-  static enum mount_status act = umounted;
-  enum mount_status ret = act;
-  if (status == umounted || status == mounted)
-    act = status;
   return ret;
 }
 
-int install_mount_proc (void)
+#ifndef MNT_DETACH
+enum { MNT_DETACH = 2 };
+#endif
+
+int install_umount (const char *what)
 {
-  if (mount_status_proc (0) == umounted)
-  {
-    char buf[PATH_MAX];
-    snprintf (buf, PATH_MAX, "%s/proc", install_root);
-    if (mount ("proc", buf, "proc", 0, 0))
-      return 1;
-    mount_status_proc (mounted);
-  }
-  return 0;
+  char buf[PATH_MAX];
+  int ret;
+
+  snprintf (buf, sizeof buf, "%s/%s", target_root, what);
+  ret = umount2 (buf, MNT_DETACH);
+
+  if (ret < 0 && errno != ENOENT && errno != EINVAL)
+    log_text (DI_LOG_LEVEL_WARNING, "Failed to unmount /%s: %s", what, strerror (errno));
+
+  return ret;
 }
 
-void install_umount (void)
+int install_umount_all (void)
 {
-  if (mount_status_proc (0) == mounted)
-  {
-    char buf[PATH_MAX];
-    snprintf (buf, PATH_MAX, "%s/proc", install_root);
-    umount (buf);
-  }
-}
-
-int install_helper_execute (const char *command)
-{
-  log_message (LOG_MESSAGE_INFO_INSTALL_HELPER_EXECUTE);
-  return execute_target (command);
+  int ret = 0;
+  ret |= install_umount ("dev");
+  ret |= install_umount ("proc");
+  return ret;
 }
 
 int install_helper_install (const char *name)
 {
-  char package[128];
-  char file_source[512];
-  char file_dest[512];
-  char buf[1024];
+  char file_source[4096];
+  char file_dest[4096];
+  char buf[4096];
   int ret;
   struct stat s;
 
-  log_message (LOG_MESSAGE_INFO_INSTALL_HELPER_INSTALL, name);
-
-  snprintf (package, sizeof (package), "%s.deb", name);
-
-  snprintf (file_source, sizeof (file_source), "%s/%s", helperdir, package);
-  snprintf (file_dest, sizeof (file_source), "%s/%s", install_root, package);
+  snprintf (file_source, sizeof file_source, "%s/%s.deb", helperdir, name);
+  snprintf (file_dest, sizeof file_dest, "%s/var/cache/bootstrap/%s.deb", target_root, name);
 
   if (stat (file_source, &s) < 0)
-    log_text (DI_LOG_LEVEL_ERROR, "Helper package %s not found.", name);
+    log_text (DI_LOG_LEVEL_ERROR, "Helper package %s not found", name);
 
-  snprintf (buf, sizeof (buf), "cp %s %s", file_source, file_dest);
+  snprintf (buf, sizeof buf, "cp %s %s", file_source, file_dest);
   ret = execute (buf);
   if (ret)
     return ret;
 
-  snprintf (buf, sizeof (buf), "dpkg -i /%s", package);
-  ret = execute_target (buf);
-  unlink (file_dest);
+  snprintf (buf, sizeof buf, "dpkg -i /var/cache/bootstrap/%s.deb", name);
+  if (simulate_handler)
+    return simulate_handler (buf);
 
-  return ret;
+  log_message (LOG_MESSAGE_INFO_INSTALL_HELPER_INSTALL, name);
+
+  return execute_target (buf);
 }
 
 int install_helper_remove (const char *name)
 {
   char buf[128];
 
+  snprintf (buf, sizeof (buf), "dpkg -P %s", name);
+  if (simulate_handler)
+    return simulate_handler (buf);
+
   log_message (LOG_MESSAGE_INFO_INSTALL_HELPER_REMOVE, name);
 
-  snprintf (buf, sizeof (buf), "dpkg -P %s", name);
-  execute_target (buf);
-
-  return 0;
+  return execute_target (buf);
 }
 
