@@ -25,9 +25,8 @@
 #include <config.h>
 
 #include "frontend.h"
-#include "install.h"
 #include "log.h"
-#include "prepare.h"
+#include "target.h"
 
 #include <debian-installer.h>
 #include <limits.h>
@@ -42,25 +41,25 @@ const log_message_text log_messages[] =
     "Couldn't decompress %s!",
     DI_LOG_LEVEL_ERROR,
   },
-  [LOG_MESSAGE_ERROR_DOWNLOAD] =
+  [LOG_MESSAGE_ERROR_DOWNLOAD_PARSE] =
+  {
+    "Couldn't parse %s!",
+    DI_LOG_LEVEL_ERROR,
+  },
+  [LOG_MESSAGE_ERROR_DOWNLOAD_RETRIEVE] =
   {
     "Couldn't download %s!",
     DI_LOG_LEVEL_ERROR,
   },
-  [LOG_MESSAGE_ERROR_INSTALL_LIST] =
+  [LOG_MESSAGE_ERROR_DOWNLOAD_VALIDATE] =
   {
-    "Couldn't not build installation list!",
+    "Couldn't validate %s!",
     DI_LOG_LEVEL_ERROR,
   },
-  [LOG_MESSAGE_ERROR_INSTALL_ROOT] =
+  [LOG_MESSAGE_WARNING_DOWNLOAD_VALIDATE] =
   {
-    "Couldn't install root!",
-    DI_LOG_LEVEL_ERROR,
-  },
-  [LOG_MESSAGE_ERROR_PARSE] =
-  {
-    "Couldn't parse %s!",
-    DI_LOG_LEVEL_ERROR,
+    "Couldn't validate %s!",
+    DI_LOG_LEVEL_WARNING,
   },
   [LOG_MESSAGE_INFO_DOWNLOAD_RETRIEVE] =
   {
@@ -77,22 +76,27 @@ const log_message_text log_messages[] =
     "Validating %s",
     DI_LOG_LEVEL_MESSAGE,
   },
-  [LOG_MESSAGE_INFO_INSTALL_DO] =
+  [LOG_MESSAGE_INFO_INSTALL_HELPER_INSTALL] =
   {
-    "Doing something important",
+    "Configuring helper %s",
     DI_LOG_LEVEL_MESSAGE,
   },
-  [LOG_MESSAGE_INFO_INSTALL_CONFIGURE_PACKAGE] =
+  [LOG_MESSAGE_INFO_INSTALL_HELPER_REMOVE] =
+  {
+    "Deconfiguring helper %s",
+    DI_LOG_LEVEL_MESSAGE,
+  },
+  [LOG_MESSAGE_INFO_INSTALL_PACKAGE_CONFIGURE] =
   {
     "Configuring package %s",
     DI_LOG_LEVEL_MESSAGE,
   },
-  [LOG_MESSAGE_INFO_INSTALL_EXTRACT_PACKAGE] =
+  [LOG_MESSAGE_INFO_INSTALL_PACKAGE_EXTRACT] =
   {
     "Extracting %s",
     DI_LOG_LEVEL_MESSAGE,
   },
-  [LOG_MESSAGE_INFO_INSTALL_UNPACK_PACKAGE] =
+  [LOG_MESSAGE_INFO_INSTALL_PACKAGE_UNPACK] =
   {
     "Unpacking package %s",
     DI_LOG_LEVEL_MESSAGE,
@@ -110,21 +114,23 @@ const char *log_levels[] =
   [DI_LOG_LEVEL_DEBUG] = "D",
 };
 
-static void message (const char *def, const char *message)
+static void message (di_log_level_flags log_level, const char *message)
 {
+  if (log_level & DI_LOG_LEVEL_DEBUG)
+    return;
+
   if (logfile)
   {
-    fprintf (logfile, "%s: %s\n", def, message);
+    fprintf (logfile, "%s: %s\n", log_levels[log_level & DI_LOG_LEVEL_MASK], message);
     fflush (logfile);
   }
 }
 
 static void handler (di_log_level_flags log_level, const char *msg, void *user_data __attribute__ ((unused)))
 {
-  if (!(log_level & DI_LOG_LEVEL_DEBUG))
-    message (log_levels[log_level & DI_LOG_LEVEL_MASK], msg);
+  message (log_level, msg);
 
-  frontend_log_text_internal (log_level, msg);
+  frontend_log_text (log_level, msg);
 
   if (log_level & DI_LOG_FATAL_MASK)
     exit (EXIT_FAILURE);
@@ -139,12 +145,13 @@ int log_init (void)
 void log_open (void)
 {
   char buf[PATH_MAX];
-  snprintf (buf, sizeof (buf), "%s/%s", install_root, "/var/log/bootstrap.log");
+
+  target_create_dir ("/var");
+  target_create_dir ("/var/log");
+
+  snprintf (buf, sizeof buf, "%s/var/log/bootstrap.log", target_root);
   if (!(logfile = fopen (buf, "w")))
-  {
-    frontend_log_text (DI_LOG_LEVEL_ERROR, "Could not open logfile!");
-    exit (EXIT_FAILURE);
-  }
+    log_text (DI_LOG_LEVEL_ERROR, "Could not open logfile");
 }
 
 void log_message (log_message_name message_name, ...)
@@ -157,7 +164,7 @@ void log_message (log_message_name message_name, ...)
   vsnprintf (buf, sizeof (buf), msg->text, args);
   va_end (args);
 
-  message (log_levels[msg->log_level], buf);
+  message (msg->log_level, buf);
 
   va_start (args, message_name);
   frontend_log_message (message_name, args);
@@ -176,12 +183,6 @@ void log_text (di_log_level_flags log_level, const char *format, ...)
   vsnprintf (msg, sizeof (msg), format, args);
   va_end (args);
 
-  if (!(log_level & DI_LOG_LEVEL_DEBUG))
-    message (log_levels[log_level & DI_LOG_LEVEL_MASK], msg);
-
-  frontend_log_text (log_level, msg);
-
-  if (log_level & DI_LOG_FATAL_MASK)
-    exit (EXIT_FAILURE);
+  handler (log_level, msg, NULL);
 }
 
